;******************************************************************************
;* CD_GPIO.S
;******************************************************************************
;
;       This is the MC68K program that creates a 'bit-banging' DSA Bus
;       interface using the GPIO bits on the Falcon030 DSP port.
;
;	Note that V105 of this code starts and stops the CD used for testing.
;	V106 is a version that tests bi-directional communications with
;	Sylvester+Butch3.
;
;	Assembled by... asm filename (lattice C assembly)
;
; 				REVISION HISTORY
;
; Rev	By    Date		Description of edit
;""""   ""    """"		"""""""""""""""""""
; 100  DMS  01 FEB 94   Original file incorporating modified CDSAMPLE.S
; 103  DMS  03 FEB 94   in-line code to TX then RX.  works
; 104  DMS  04 FEB 94   make it receive 100 words
; 105  DMS  04 FEB 94   clean it up and make it modular
; 106  DMS  07 FEB 94   Adapt it for Sylvester+Butch3 communications
; 108  DMS  08 FEB 94   fix end of RX comm phase
; 109  DMS  08 FEB 94   V108 works (w/ ButchDSA.102).  Now make Falcon RX 1st
; 110  DMS  08 FEB 94   Put polling loop in main line of code
; 111  PKF  08 FEB 94	Modified for Lattice Assembler
;
; 112  PKF  11 FEB 94	added a _BS_DSA routine, which checks if the DSA bus
;			is busy ?
;
; 113  PKF  15 FEB 94	added a routine which reads ISR register at the
;			host interface port and returns the ISR value 
;******************************************************************************

;******************************************************************************
;*	COMMON EQUATES
;******************************************************************************

;------------------------------------------------------------------------------
; GENERAL PURPOSE REGISTERS :
;
; These addresses are memory mapped registers for general purpose io.
;
; In general, the theory is you should write data to the port
; and then enable pins for direction(s) desired.
;
;------------------------------------------------------------------------------
					;
G_contrl	equ	$FF8940		; R 0000Xddd and 0000Xccc if c=1, pin is
					; an output.  ddd is the state of the bits.
					;
					; W XXXXXXXX and XXXXXccc set c=1 for output
					;
					; Bit 0 : Control line for DATA
					; Bit 1 : Control line for STROBE
					; Bit 2 : Control line for ACKNOWLEDGE
					;
G_data		equ	$FF8942		; R 0000Xddd and 0000Xiii i is the pin state
					;
					; W XXXXXXXX and XXXXXddd d is the data
					;
					; Bit 0 : DATA line
					; Bit 1 : STROBE line
					; Bit 2 : ACKNOWLEDGE line
					;
;------------------------------------------------------------------------------
; HOST INTERFACE REGISTERS :
;
; following are the DSP Host Processor interface registers used for
; transferring data from host processor to the DSP
;
;------------------------------------------------------------------------------
					;
ICR		equ	$ffffa200	; interrupt control register
CVR		equ	$ffffa201	; command vector rsgieter
ISR		equ	$ffffa202	; interrupt status register
IVR		equ	$ffffa203	; interrupt vector register

RLONG		equ	$ffffa204	; receive long
RXH		equ	$ffffa205	; receive high byte
RXM		equ	$ffffa206	; receive middle byte
RXL		equ	$ffffa207	; receive low byte

TLONG		equ	$ffffa204	; transmit long
TXH		equ	$ffffa205	; transmit high byte
TXM		equ	$ffffa206	; transmit middle byte
TXL		equ	$ffffa207	; transmit low byte
					;
;******************************************************************************
;* DATA ACCESSED FROM OTHER MODULES
;******************************************************************************

	XDEF	_DSA_in
	XDEF	_DSA_out
	XDEF	_DSA_err

;******************************************************************************
;*	CODE SECTION
;******************************************************************************

	CSECT	text,code

;******************************************************************************
;* ROUTINES ACCESSED FROM OTHER MODULES
;******************************************************************************

	XDEF	_CH_DSA
	XDEF	_BS_DSA
	XDEF	_TX_DSA
	XDEF	_RX_DSA

	XDEF	_GT_ISR:

;******************************************************************************
;******************************************************************************

;||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
;				SUBROUTINES	
;||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

;					***************************************
;					* Checks DSA bur for DATA low	      *
;					***************************************

_CH_DSA:
	clr.l	d0			; clear d0
	move.w	G_data,d0		; get data into d0
	btst	#0,d0			; Jaguar wants to send something ?
	andi.w	#1,d0			; mask off data bit
	rts

;					***************************************
;					* Checks for DSA free		      *
;					***************************************

_BS_DSA:

	clr.l	d0
	move.w	G_data,d0		; get DSA ACK in d0
	cmp.w	#$0707,d0		; check if bus free
	bne	retfalse
	move.w	#1,d0
	rts				; return BUS FREE

retfalse:
	move.w	#0,d0
	rts				; return BUS BUSY

;					***************************************
;					* DSA Bus Write Cycle		      *
;					***************************************
_TX_DSA:
	movem.l	d0-d3,-(sp)		; save registers

;----------------------------------------
; Initialise data
;----------------------------------------

	move.w	#0,_DSA_err		; clear DSA error flag
	move.w	_DSA_out,d0		; load word to be transmitted in d0
	clr.l	d2			; bit counter

;----------------------------------------
; Start synchronization
;----------------------------------------

DAT_000:
	move.w	#0,G_data		; set DATA line lo
	move.w	#1,G_contrl		; enable output on DATA line
	move.w	#$FFFF,d3		; initialise d3 for time out
ACK_000:				; wait for DSA ACK to go lo
	move.w	G_data,d1		; get DSA ACK in d1
	btst	#2,d1			; is DSA ACK lo ?
	beq	DAT_001			; yes, proceed
	dbra	d3,ACK_000		; nope check if timeout
	move.w	#$F000,_DSA_err		; timed out,
	bra	Fini			; set error code and bail out
DAT_001:				; DSA ACK was lowered by CD
	move.w	#1,G_data		; set DATA line hi
	move.w	#1,G_contrl		; enable output on DATA line
	move.w	#$FFFF,d3		; initialise d3 for time out
ACK_001:				; wait for DSA ACK to go hi
	move.w	G_data,d1		; get DSA ACK in d1
	btst	#2,d1			; is DSA ACK hi ?
	bne	senddata		; yes, proceed
	dbra	d3,ACK_001		; nope, check if timeout
	move.w	#$F001,_DSA_err		; timed out,
	bra	Fini			; set error code and bail out

;----------------------------------------
; Start data Xmit
;----------------------------------------

senddata:
	move.w	#7,G_data		; first set all lines to high
	move.w	#0,G_contrl		; and set them for input

testbit:
	btst	#15,d0			; see if bit 15 is 1 or 0
	bne	send1			; it was a 1 so jump, else fall through

;----------------------------------------
; Transmit a '0'
;----------------------------------------

send0:
	move.w	#2,G_data		; set STROBE line lo and DATA line lo
	move.w	#3,G_contrl		; enable DATA line to output lo
	move.w	#2,d1			; put a delay before we make STROBE lo
STR_010:
	dbra	d1,STR_010		; delay before STROBE lo
	move.w	#0,G_data		; set DATA line to lo
	move.w	#3,G_contrl		; enable STROBE line to output lo
	move.w	#$FFFF,d3		; initialise d3 for time out
ACK_010:
	move.w	G_data,d1		; get DSA ACK in d1
	btst	#2,d1			; is DSA ACK lo ?
	beq	STR_011			; yes, proceed
	dbra	d3,ACK_010		; nope, check if timeout
	move.w	#$F002,_DSA_err		; timed out,
	bra	Fini			; set error code and bail out
STR_011:
	move.w	#2,G_data		; set STROB line hi and DATA line lo
	move.w	#3,G_contrl		; set STROBE and DATA lines for output
	move.w	#$FFFF,d3		; initialise d3 for time out
ACK_011:				; wait for DSA ACK to go hi
	move.w	G_data,d1		; get data register into d1
	btst	#2,d1			; is DSA ACK hi ?
	bne	nextbit			; yes, proceed with next bit
	dbra	d3,ACK_011		; nope, check if timeout
	move.w	#$F003,_DSA_err		; timed out,
	bra	Fini			; set error code and bail out

;----------------------------------------
; Transmit a '1'
;----------------------------------------

send1:
	move.w	#3,G_data		; set STROBE line lo and DATA line hi
	move.w	#3,G_contrl		; enable DATA line to output hi
	move.w	#2,d1			; put a delay before we make STROBE lo
STR_020:
	dbra	d1,STR_020		; delay before STROBE lo
	move.w	#1,G_data		; set data line to hi
	move.w	#3,G_contrl		; enable STROBE line to output lo
	move.w	#$FFFF,d3		; initialise d3 for time out
ACK_020:
	move.w	G_data,d1		; get DSA ACK in d1
	btst	#2,d1			; is DSA ACK lo ?
	beq	STR_021			; yes, proceed
	dbra	d3,ACK_020		; nope, check if timeout
	move.w	#$F004,_DSA_err		; timed out,
	bra	Fini			; set error code and bail out
STR_021:				; OK, ACK is lo,
	move.w	#3,G_data		; set STROB line hi and DATA line hi
	move.w	#3,G_contrl		; set STROBE and DATA lines for output
	move.w	#$FFFF,d3		; initialise d3 for time out
ACK_021:				; wait for DSA ACK to go hi
	move.w	G_data,d1		; get data register into d1
	btst	#2,d1			; is DSA ACK hi ?
	bne	nextbit			; yes, proceed with next bit
	dbra	d3,ACK_021		; nope, check if timeout
	move.w	#$F005,_DSA_err		; timed out,
	bra	Fini			; set error code and bail out

;----------------------------------------
; check if all 16 bits sent already
;----------------------------------------

nextbit:
	move.w	#7,G_data		; release the DSA bus
	move.w	#0,G_contrl		; after each bit is sent

	add.w	#1,d2			; bit was received, so increment bit-counter
	cmp.w	#$10,d2			; see if we're done sending 16 bits
	beq	senddone		; goto TX confirmation phase if done here
	rol.w	#1,d0			; not done, shift to get next bit
	bra	testbit			; return for more bits

;----------------------------------------
; ALL 16 data bits are sent
;----------------------------------------

senddone:
	move.w	#7,G_data		; now, let's try and get confirmation
	move.w	#0,G_contrl		; but first, release the bus

;----------------------------------------
; Start Xmit confirm phase
;----------------------------------------

ACK_030:
	move.w	#0,G_data		; set ACK line lo
	move.w	#4,G_contrl		; enable ACK for output
	move.w	#$FFFF,d3		; initialise d3 for time out
STR_030:				; watching for strobe lo
	move.w	G_data,d1		; get STROBE in d1
	btst	#1,d1			; is DSA STROB lo ?
	beq	DAT_030			; yes, proceed
	dbra	d3,STR_030		; nope, check if timeout
	move.w	#$F006,_DSA_err		; timed out,
	bra	Fini			; set error code and bail out
DAT_030:
	move.w	G_data,d1		; get data line
	btst	#0,d1			; is DSA DATA hi ?
	bne	ACK_031			; yes, proceed
	move.w	#$F100,_DSA_err		; error on transmit (a '0' on DATA line)
	bra	Fini			; set error code and bail out
ACK_031:				; no, CD set data hi, so...
	move.w	#4,G_data		; we set ACK hi also, to handshake
	move.w	#4,G_contrl		; sync completed
	bra	Fini			; successful, so bail out


;					***************************************
;					* Receive Word From CD Module	      *
;					***************************************
_RX_DSA:				; do the receive synch stuff first 
	movem.l	d0-d3,-(sp)		; save registers

;----------------------------------------
; Initialise data
;----------------------------------------

	move.w	#0,_DSA_err		; clear DSA error flag
	clr.l	d0			; assemble incomming word here
	clr.l	d2			; bit-counter

;----------------------------------------
; Start synchronization phase
;----------------------------------------

ACK_100:
	move.w	#0,G_data		; acknowledge that we saw data lo
	move.w	#4,G_contrl		; send ACK lo
	move.w	#$FFFF,d3		; initialise d3 for time out
DAT_100:				; wait for data line to go hi
	move.w	G_data,d1		; get data line
	btst	#0,d1			; id DSA DATA hi ?
	bne	ACK_101			; yes, it is hi, proceed
	dbra	d3,DAT_100		; nope, check if timeout
	move.w	#$F010,_DSA_err		; timed-out.
	bra	Fini			; set error code and exit
ACK_101:				; OK, DATA hi,
	move.w	#4,G_data		; so we acknowledge with ACK hi
	move.w	#4,G_contrl		; synchronozation completed

;----------------------------------------
; start actual receive process for 1 word
;----------------------------------------

readbit:
	move.w	#$FFFF,d3		; initialise d3 for time out
STR_200:				; watching for strobe lo
	move.w	G_data,d1		; get STROBE in d1
	btst	#1,d1			; is DSA STROB lo ?
	beq	maskbit			; yes, proceed
	dbra	d3,STR_200		; nope, check if timeout
	move.w	#$F011,_DSA_err		; timed-out.
	bra	Fini			; set error code and exit
maskbit:
	andi.w	#1,d1			; mask out data from d1
	add.w	d1,d0			; read data to d0
	rol.w	#1,d0			; move it over 1 bit
	add.w	#1,d2			; increment bit-counter
ACK_200:
	move.w	#0,G_data		; set ACK to lo
	move.w	#4,G_contrl		; send ACK lo on DSA bus
	move.w	#$FFFF,d3		; initialise d3 for time out
STR_201:
	move.w	G_data,d1		; get STROBE line
	btst	#1,d1			; is DSA strobe hi ?
	bne	ACK_201			; yes, proceed
	dbra	d3,STR_201		; nope, check if timeout
	move.w	#$F012,_DSA_err		; timed-out.
	bra	Fini			; set error code and exit
ACK_201:
	move.w	#4,G_data		; OK, strobe went hi, so set ACK hi 
	move.w	#4,G_contrl		; to complete the cycle for 1 bit

;----------------------------------------
; check if we received all 16 bits
;----------------------------------------

savebit:
	cmp.w	#$10,d2			; have we received 16 bits yet ?
	bne	readbit			; not yet, jump to keep going
	ror.w	#1,d0			; shift word right to adjust for mask, above in 'maskbit'
	move.w	d0,_DSA_in		; save word in named location

;----------------------------------------
; Start confirmation phase
;----------------------------------------

confirm:
	move.w	#7,G_data		; confirm to CD that we got the word
	move.w	#0,G_contrl		; release bus with data hi to show RX was OK
	move.w	#$FFFF,d3		; initialise d3 for time out
ACK_300:
	move.w	G_data,d1		; get ACK in d1
	btst	#2,d1			; is DSA ACK lo ?
	beq	STR_300			; yes, proceed
	dbra	d3,ACK_300		; nope, check if timeout
	move.w	#$F013,_DSA_err		; timed-out.
	bra	Fini			; set error code and exit
STR_300:
	move.w	#1,G_data		; OK, it's lo, so make STROBE lo, DATA hi
	move.w	#3,G_contrl
	move.w	#$FFFF,d3		; initialise d3 for time out
ACK_301:
	move.w	G_data,d1		; get data register 
	btst	#2,d1			; is DSA ACK hi ?
	bne	SayRX_OK		; yes, proceed
	dbra	d3,ACK_301		; nope, check if timeout
	move.w	#$F014,_DSA_err		; timed-out.
	bra	Fini			; set error code and exit
SayRX_OK:				; OK, ACK is hi, so release the bus
	move.w	#3,G_data		; make data and strobe hi to handshake
	move.w	#3,G_contrl		; confirm phase is done
	move.w	#100,d3			; wait, so sender can see the wires
Hold_it1:
	dbra	d3,Hold_it1
	move.w	#0,G_contrl		; now release bus.  RX is done

Fini:
	move.w	#7,G_data
	move.w	#0,G_contrl		; release bus by making it all inputs

	movem.l	(sp)+,d0-d3		; restore registers
	rts				; return to caller

;******************************************************************************
; CODE TO HANDLE HOST INTERFACE
;******************************************************************************

_GT_ISR:

	clr.l	d0			; clear register d0
	move.b	ISR,d0			; get the interrupt register in d0.b
	rts

;******************************************************************************
;*	BSS SECTION
;******************************************************************************

	CSECT	xxxx,bss

;******************************************************************************
;******************************************************************************

_DSA_in:	ds.w	1
_DSA_out:	ds.w	1
_DSA_err:	ds.w	1

;******************************************************************************
;* END OF PRAGRAM
;******************************************************************************

	END

;******************************************************************************
;* END OF CD_GPIO.S
;******************************************************************************

